<?php

/**
 * Displays the registration page.
 */
function auth_controller_register_get(): void
{
	gate(false, false);
	echo render('layouts/basic', ['view' => 'pages/auth/register']);
}

/**
 * Handles the registration form submission.
 */
function auth_controller_register_post(): void
{
	gate(false, false);
	csrf_ensure();

	$errors = [];

	$u = $_POST['username'] ?? '';
	$e = $_POST['email'] ?? '';
	$p = $_POST['password'] ?? '';

	// Trim the input.
	$u = trim($u);
	$e = trim($e);

	/*
		A username is required.
		A username must be at least 3 characters long and at most 18 characters long.
		A username must contain only alphanumeric characters and spaces.
	*/
	if (empty($u) || strlen($u) < 3 || strlen($u) > 18 || !ctype_alnum(str_replace(' ', '', $u))) {
		$errors['u'][] = 'Username is required and must be between 3 and 18 characters long and contain only
			alphanumeric characters and spaces.';
	}

	/*
		An email is required.
		An email must be at most 255 characters long.
		An email must be a valid email address.
	*/
	if (empty($e) || strlen($e) > 255 || !filter_var($e, FILTER_VALIDATE_EMAIL)) {
		$errors['e'][] = 'Email is required must be a valid email address.';
	}

	/*
		A password is required.
		A password must be at least 6 characters long.
	*/
	if (empty($p) || strlen($p) < 6) {
		$errors['p'][] = 'Password is required and must be at least 6 characters long.';
	}

	// If there are errors at this point, send them to the page with errors flashed.
	if (!empty($errors)) {
		flash('errors', $errors);
		redirect('/auth/register');
	}

	/*
		A username must be unique.
	*/
	if (auth_usernameExists($u)) {
		$errors['u'][] = 'Username is already taken.';
	}

	/*
		An email must be unique.
	*/
	if (auth_emailExists($e)) {
		$errors['e'][] = 'Email is already taken.';
	}

	// If there are errors at this point, send them to the page with errors flashed.
	if (!empty($errors)) {
		flash('errors', $errors);
		redirect('/auth/register');
	}

	$user = user_create($u, $e, $p);
	if ($user === false) router_error(400);

	$_SESSION['user'] = user_find($u);
	redirect('/character/create-first');
}

/**
 * Displays the login page.
 */
function auth_controller_login_get(): void
{
	gate(false, false);
	echo render('layouts/basic', ['view' => 'pages/auth/login']);
}

/**
 * Handles the login form submission.
 */
function auth_controller_login_post(): void
{
	gate(false, false);
	csrf_ensure();

	$errors = [];

	$u = $_POST['username'] ?? '';
	$p = $_POST['password'] ?? '';

	// Trim the input.
	$u = trim($u);

	/*
		A username is required.
	*/
	if (empty($u)) {
		$errors['u'][] = 'Username is required.';
	}

	/*
		A password is required.
	*/
	if (empty($p)) {
		$errors['p'][] = 'Password is required.';
	}

	// If there are errors at this point, send them to the page with errors flashed.
	if (!empty($errors)) {
		flash('errors', $errors);
		redirect('/auth/login');
	}

	$user = user_find($u);
	if ($user === false || !password_verify($p, $user['password'])) {
		$errors['u'][] = 'Invalid username or password.';
		flash('errors', $errors);
		redirect('/auth/login');
	}

	$_SESSION['user'] = $user;
	$_SESSION['char'] = char_find($user['char_id']);
	if ($_POST['remember'] ?? false) auth_rememberMe();
	redirect('/');
}

/**
 * Logs the user out.
 */
function auth_controller_logout_post(): void
{
	csrf_ensure();
	session_delete($_SESSION['user']['id']);
	unset($_SESSION['user']);
	unset($_SESSION['char']);
	set_cookie('remember_me', '', 1);
	redirect('/');
}

/**
 * Changes the user's currently selected character.
 */
function auth_controller_change_character_post(): void
{
	gate(true);
	csrf_ensure();

	$char_id = (int) ($_POST['char_id'] ?? 0);
	if (char_exists($char_id) === false) router_error(400);

	$_SESSION['user']['char_id'] = $char_id;
	if (db_query(db_auth(), 'UPDATE users SET char_id = :c WHERE id = :u', [
		':c' => $char_id,
		':u' => $_SESSION['user']['id']
	]) === false) router_error(400);

	$_SESSION['char'] = char_find($char_id);

	redirect('/');
}
